<?php

/* Icinga Dependencies Web | (c) 2024 Icinga GmbH | AGPLv3 */

namespace Icinga\Module\Dependencies\Data;

use Icinga\Module\Icingadb\Redis\VolatileStateResults;
use Traversable;
use Generator;

class UnreachableParentResults extends VolatileStateResults
{
    /** @var array<string, array<int, array<int, object>>> The root problem paths */
    protected $paths = [];

    /**
     * Get the root problem paths for given root problem id
     *
     * @param string $rootProblemId
     *
     * @return array<int, array<int, object>>
     */
    public function getPaths(string $rootProblemId): array
    {
        return $this->paths[$rootProblemId] ?? [];
    }

    protected function yieldTraversable(Traversable $traversable): Generator
    {
        $rootProblems = [];
        $allPaths = [];
        $knownIds = [];
        foreach ($traversable as $value) {
            if (
                ! $value->is_group_member
                && (
                    ($value->service_id && (
                        $value->service->state->affects_children
                        || (! $value->_has_problematic_parent && ! $value->service->state->is_reachable)
                    ))
                    || ($value->service_id === null && $value->host->state->affects_children)
                    || ($value->redundancy_group->state->failed && $value->redundancy_group->state->is_reachable)
                )
            ) {
                if (empty($value->redundancy_group->id) && empty($value->host->id) && empty($value->service->id)) {
                    // objects that cannot be fetched due to restrictions
                    continue;
                }

                if (! isset($knownIds[$value->id])) {
                    $knownIds[$value->id] = $value;

                    yield $value;
                }

                $rootProblems[] = $value;
            } else {
                $allPaths[$value->id] = $value;
            }
        }

        foreach ($rootProblems as $rootProblem) {
            $this->paths[$rootProblem->id][] = $this->buildPath($rootProblem->child_id, $allPaths);
        }
    }

    /**
     * Build the path from the root problem object to the current object
     *
     * @param $childId string The child id of the (root) problem object
     * @param $allPaths array<string, object> All objects that are not root problems
     * @param $pathChain array<int, object> The path chain from the root problem object to the current object
     *
     * @return array<int, object>
     */
    private function buildPath(string $childId, array &$allPaths, array &$pathChain = []): array
    {
        if (! isset($allPaths[$childId])) {
            return $pathChain;
        }

        $child = $allPaths[$childId];
        $pathChain[] = $child;

        return $this->buildPath($child->child_id, $allPaths, $pathChain);
    }
}
