<?php

/* Icinga Dependencies Web | (c) 2024 Icinga GmbH | AGPLv3 */

namespace Icinga\Module\Dependencies\Model;

use Icinga\Module\Icingadb\Model\DependencyNode;
use ipl\Orm\Query;
use ipl\Sql\Connection;
use ipl\Sql\Expression;

/**
 * @property int $nodes_ok
 * @property int $nodes_problem_handled Only include hosts and services
 * @property int $nodes_problem_unhandled Include hosts, services and redundancy groups
 * @property int $nodes_warning_handled
 * @property int $nodes_warning_unhandled
 * @property int $nodes_unknown_handled
 * @property int $nodes_unknown_unhandled
 * @property int $nodes_pending
 * @property int $nodes_total
 */
class DependencyNodeSummary extends DependencyNode
{
    public function getSummaryColumns(): array
    {
        return [
            'nodes_ok' => new Expression(
                'SUM(CASE'
                . ' WHEN %s IS NOT NULL THEN (CASE WHEN %s = 0 THEN 1 ELSE 0 END)'
                . ' WHEN %s IS NOT NULL THEN (CASE WHEN %s = 0 THEN 1 ELSE 0 END)'
                . " ELSE (CASE WHEN %s = 'n' THEN 1 ELSE 0 END)"
                . ' END)',
                [
                    'service_id',
                    'service.state.soft_state',
                    'host_id',
                    'host.state.soft_state',
                    'redundancy_group.state.failed'
                ]
            ),
            'nodes_problem_handled' => new Expression(
                'SUM(CASE'
                . " WHEN %s IS NOT NULL THEN (CASE WHEN %s = 2 AND (%s = 'y' OR %s = 'n') THEN 1 ELSE 0 END)"
                . " WHEN %s IS NOT NULL THEN (CASE WHEN %s = 1 AND (%s = 'y' OR %s = 'n') THEN 1 ELSE 0 END)"
                . ' END)',
                [
                    'service_id',
                    'service.state.soft_state',
                    'service.state.is_handled',
                    'service.state.is_reachable',
                    'host_id',
                    'host.state.soft_state',
                    'host.state.is_handled',
                    'host.state.is_reachable'
                ]
            ),
            'nodes_problem_unhandled' => new Expression(
                'SUM(CASE'
                . " WHEN %s IS NOT NULL THEN (CASE WHEN %s = 2 AND %s = 'n' AND %s = 'y' THEN 1 ELSE 0 END)"
                . " WHEN %s IS NOT NULL THEN (CASE WHEN %s = 1 AND %s = 'n' AND %s = 'y' THEN 1 ELSE 0 END)"
                . " ELSE (CASE WHEN %s = 'y' THEN 1 ELSE 0 END)"
                . ' END)',
                [
                    'service_id',
                    'service.state.soft_state',
                    'service.state.is_handled',
                    'service.state.is_reachable',
                    'host_id',
                    'host.state.soft_state',
                    'host.state.is_handled',
                    'host.state.is_reachable',
                    'redundancy_group.state.failed'
                ]
            ),
            'nodes_warning_handled' => new Expression(
                "SUM(CASE WHEN %s IS NOT NULL AND %s = 1 AND (%s = 'y' OR %s = 'n') THEN 1 ELSE 0 END)",
                [
                    'service_id',
                    'service.state.soft_state',
                    'service.state.is_handled',
                    'service.state.is_reachable'
                ]
            ),
            'nodes_warning_unhandled' => new Expression(
                "SUM(CASE WHEN %s IS NOT NULL AND %s = 1 AND %s = 'n' AND %s = 'y' THEN 1 ELSE 0 END)",
                [
                    'service_id',
                    'service.state.soft_state',
                    'service.state.is_handled',
                    'service.state.is_reachable'
                ]
            ),
            'nodes_unknown_handled' => new Expression(
                "SUM(CASE WHEN %s IS NOT NULL AND %s = 3 AND (%s = 'y' OR %s = 'n') THEN 1 ELSE 0 END)",
                [
                    'service_id',
                    'service.state.soft_state',
                    'service.state.is_handled',
                    'service.state.is_reachable'
                ]
            ),
            'nodes_unknown_unhandled' => new Expression(
                "SUM(CASE WHEN %s IS NOT NULL AND %s = 3 AND %s = 'n' AND %s = 'y' THEN 1 ELSE 0 END)",
                [
                    'service_id',
                    'service.state.soft_state',
                    'service.state.is_handled',
                    'service.state.is_reachable'
                ]
            ),
            'nodes_pending' => new Expression(
                'SUM(CASE'
                . ' WHEN %s IS NOT NULL THEN (CASE WHEN %s = 99 THEN 1 ELSE 0 END)'
                . ' WHEN %s IS NOT NULL THEN (CASE WHEN %s = 99 THEN 1 ELSE 0 END)'
                . ' END)',
                [
                    'service_id',
                    'service.state.soft_state',
                    'host_id',
                    'host.state.soft_state'
                ]
            ),
            'nodes_total' => new Expression('COUNT(*)'),
        ];
    }

    public static function on(Connection $db): Query
    {
        $q = parent::on($db);

        /** @var static $m */
        $m = $q->getModel();
        $q->columns($m->getSummaryColumns());

        return $q;
    }

    public function getColumns(): array
    {
        return array_merge(parent::getColumns(), $this->getSummaryColumns());
    }

    public function getDefaultSort(): array
    {
        return [];
    }
}
