<?php

/* Icinga Dependencies Web | (c) 2024 Icinga GmbH | AGPLv3 */

namespace Icinga\Module\Dependencies\ProvidedHook\Icingadb;

use Icinga\Module\Dependencies\Widget\ItemList\RootProblemDependencyNodeList;
use Icinga\Module\Icingadb\Common\Backend;
use Icinga\Module\Icingadb\Hook\HostDetailExtensionHook;
use Icinga\Module\Icingadb\Model\Host;
use ipl\Html\Attributes;
use ipl\Html\HtmlDocument;
use ipl\Html\HtmlElement;
use ipl\Html\Text;
use ipl\Html\ValidHtml;
use ipl\I18n\Translation;
use Icinga\Module\Dependencies\Common\RootProblemPathQueryUtils;

class HostRootProblemExtension extends HostDetailExtensionHook
{
    use Translation;
    use RootProblemPathQueryUtils;

    protected function init(): void
    {
        $this->setLocation(0);
    }

    public function getHtmlForObject(Host $object): ValidHtml
    {
        if ($object->state->is_reachable || ! Backend::supportsDependencies()) {
            return new HtmlDocument();
        }

        $rootProblems = $this->createQuery($object);

        return (new HtmlDocument())
            ->addHtml(
                new HtmlElement(
                    'h2',
                    Attributes::create(['class' => 'root-problems-with-path']),
                    Text::create($this->translate('Root Problems'))
                ),
                (new RootProblemDependencyNodeList($rootProblems->execute(), $object))
                    ->setEmptyStateMessage($this->translate('You are not authorized to view these objects.'))
            );
    }
}
