<?php

/* Icinga Dependencies Web | (c) 2024 Icinga GmbH | AGPLv3 */

namespace Icinga\Module\Dependencies\Widget\ItemList;

use Icinga\Exception\NotImplementedError;
use Icinga\Module\Dependencies\Data\UnreachableParentResults;
use Icinga\Module\Dependencies\Model\UnreachableParentWithPath;
use Icinga\Module\Dependencies\View\RootProblemHostRenderer;
use Icinga\Module\Dependencies\View\RootProblemRedundancyGroupRenderer;
use Icinga\Module\Dependencies\View\RootProblemServiceRenderer;
use Icinga\Module\Icingadb\Model\Host;
use Icinga\Module\Icingadb\Model\RedundancyGroup;
use Icinga\Module\Icingadb\Model\Service;
use Icinga\Module\Icingadb\Widget\ItemList\ObjectList;
use ipl\Orm\Model;
use ipl\Web\Layout\DetailedItemLayout;
use ipl\Web\Widget\ItemList;

/**
 * Create Dependency node list with the path to the root problem object
 *
 * @extends ObjectList<UnreachableParentWithPath, Host|Service|RedundancyGroup>
 */
class RootProblemDependencyNodeList extends ObjectList
{
    protected $defaultAttributes = ['class' => ['root-problem-dependency-node-list']];

    /**
     * RootProblemDependencyNodeList constructor.
     *
     * @param UnreachableParentResults $data The result set
     * @param Host|Service|RedundancyGroup $currentObject The current object on which the query is based
     */
    public function __construct(UnreachableParentResults $data, Model $currentObject)
    {
        ItemList::__construct($data, function ($item) use ($currentObject) {
            if ($item instanceof RedundancyGroup) {
                $renderer = new RootProblemRedundancyGroupRenderer();
            } elseif ($item instanceof Service) {
                $renderer = new RootProblemServiceRenderer();
            } elseif ($item instanceof Host) {
                $renderer = new RootProblemHostRenderer();
            } else {
                throw new NotImplementedError('Not implemented');
            }

            return $renderer
                ->setCurrentObject($currentObject)
                ->setData($this->data);
        });

        $this->setItemLayoutClass(DetailedItemLayout::class);
    }
}
